@@ -1,7 +1,7 @@
 use anyhow::Result;
 use rsdns::{
+    clients::{ClientConfig, ProtocolStrategy, Recursion},
     constants::Type,
-    resolvers::{ProtocolStrategy, Recursion, ResolverConfig},
 };
 use std::{
     net::{IpAddr, SocketAddr},
@@ -16,8 +16,15 @@ pub mod bi {
     include!(concat!(env!("OUT_DIR"), "/built.rs"));
 }
 
+#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+pub enum OutputFormat {
+    Zone,
+    Short,
+    Rust,
+}
+
 #[derive(Debug, StructOpt)]
-#[structopt(about = "DNS Stub Resolver", version = env!("CH4_VERSION"))]
+#[structopt(about = "DNS Client", version = env!("CH4_VERSION"))]
 pub struct Args {
     #[cfg(all(target_os = "linux", feature = "net-tokio", feature = "socket2"))]
     #[structopt(short, long)]
@@ -45,12 +52,21 @@ pub struct Args {
     #[structopt(long, help = "Prints build information")]
     info: bool,
 
-    #[structopt(skip)]
-    pub short: bool,
-
     #[structopt(long, help = "Lists system nameservers")]
     list_nameservers: bool,
 
+    #[structopt(skip)]
+    pub format: OutputFormat,
+
+    #[structopt(skip)]
+    pub config: ClientConfig,
+
+    #[structopt(skip)]
+    pub qtype: Option<Type>,
+
+    #[structopt(skip)]
+    pub qnames: Vec<String>,
+
     #[structopt(verbatim_doc_comment)]
     /// Positional arguments ...
     ///
@@ -84,12 +100,15 @@ pub struct Args {
     /// +[no]short    - enables (disables) short output.
     ///                 When enabled, only record data is printed,
     ///                 one record on a line.
+    ///
+    /// +[no]rust     - enables (disables) rust output.
+    ///                 When enabled, prints the response as a Rust array.
     pub positional: Vec<String>,
 }
 
 impl Args {
     pub fn get() -> Result<Args> {
-        let args = Args::from_args();
+        let mut args = Args::from_args();
 
         if args.info {
             Args::show_info();
@@ -101,9 +120,15 @@ impl Args {
             exit(0);
         }
 
+        args.parse()?;
+
         Ok(args)
     }
 
+    pub fn qtype(&self) -> Type {
+        self.qtype.unwrap()
+    }
+
     fn show_info() {
         println!("build time:          {}", bi::BUILT_TIME_UTC);
         println!("ch4 semver:          {}", bi::PKG_VERSION);
@@ -135,13 +160,13 @@ impl Args {
         Ok(())
     }
 
-    pub fn parse(&mut self) -> Result<(ResolverConfig, Type, Vec<String>)> {
+    fn parse(&mut self) -> Result<()> {
         let mut protocol_strategy = ProtocolStrategy::Udp;
         let mut nameserver_ip_addr: Option<IpAddr> = None;
         let mut recursion = Recursion::On;
-        let mut short = false;
-        let mut free_args = Vec::new();
+        let mut qnames = Vec::new();
         let mut qtype = Type::A;
+        let mut format = OutputFormat::Zone;
 
         for a in self.positional.iter() {
             match a.as_str() {
@@ -150,8 +175,10 @@ impl Args {
                 "+notcp" => protocol_strategy = ProtocolStrategy::NoTcp,
                 "+rec" => recursion = Recursion::On,
                 "+norec" => recursion = Recursion::Off,
-                "+short" => short = true,
-                "+noshort" => short = false,
+                "+short" => format = OutputFormat::Short,
+                "+noshort" => format = OutputFormat::Zone,
+                "+rust" => format = OutputFormat::Rust,
+                "+norust" => format = OutputFormat::Zone,
                 s if s.starts_with('@') => match IpAddr::from_str(&s[1..]) {
                     Ok(addr) => nameserver_ip_addr = Some(addr),
                     Err(_) => {
@@ -162,11 +189,11 @@ impl Args {
                 s if Type::from_str(&s.to_uppercase()).is_ok() => {
                     qtype = Type::from_str(&s.to_uppercase()).unwrap()
                 }
-                _ => free_args.push(a.clone()),
+                _ => qnames.push(a.clone()),
             }
         }
 
-        self.short = short;
+        self.format = format;
 
         if !qtype.is_data_type() && qtype != Type::Any {
             eprintln!("only data-type queries are supported or ANY: {}", qtype);
@@ -186,7 +213,7 @@ impl Args {
         };
 
         #[allow(unused_mut)]
-        let mut conf = ResolverConfig::with_nameserver(nameserver)
+        let mut config = ClientConfig::with_nameserver(nameserver)
             .set_protocol_strategy(protocol_strategy)
             .set_recursion(recursion)
             .set_query_timeout(if self.query_timeout > 0 {
@@ -198,9 +225,25 @@ impl Args {
 
         #[cfg(all(target_os = "linux", feature = "net-tokio", feature = "socket2"))]
         if let Some(ref bd) = self.bind_device {
-            conf = conf.set_bind_device(Some(bd))?;
+            config = config.set_bind_device(Some(bd))?;
         }
 
-        Ok((conf, qtype, free_args))
+        self.config = config;
+        self.qtype = Some(qtype);
+        self.qnames = qnames;
+
+        Ok(())
+    }
+}
+
+impl Default for OutputFormat {
+    fn default() -> Self {
+        OutputFormat::Zone
+    }
+}
+
+impl OutputFormat {
+    pub fn is_short(self) -> bool {
+        self == OutputFormat::Short
     }
 }
